// -----------------------------------------------------------------------------

__device__ __inline__ float3 closest_hit_radiance_impl()
{
  // intersection vectors
  const float3 n = normalize(rtTransformNormal(RT_OBJECT_TO_WORLD, shading_normal)); // normal
  const float3 fhp = rtTransformPoint(RT_OBJECT_TO_WORLD, front_hit_point);
  const float3 bhp = rtTransformPoint(RT_OBJECT_TO_WORLD, back_hit_point);
  const float3 i = ray.direction;                                            // incident direction
        float3 t;                                                            // transmission direction
        float3 r;                                                            // reflection direction

  float reflection = 1.0f;
  float3 result = make_float3(0.0f);
  
  const int depth = prd_radiance.depth;

  float3 beer_attenuation;
  if(dot(n, ray.direction) > 0) {
    // Beer's law attenuation
    beer_attenuation = exp(extinction_constant * t_hit);
  } else {
    beer_attenuation = make_float3(1);
  }

  // refraction
  if (depth < min(refraction_maxdepth, max_depth))
  {
    if ( refract(t, i, n, refraction_index) )
    {
      // check for external or internal reflection
      float cos_theta = dot(i, n);
      if (cos_theta < 0.0f)
        cos_theta = -cos_theta;
      else
        cos_theta = dot(t, n);

      reflection = fresnel_schlick(cos_theta, fresnel_exponent, fresnel_minimum, fresnel_maximum);

      float importance = prd_radiance.importance * (1.0f-reflection) * optix::luminance( refraction_color * beer_attenuation );
      float3 color = cutoff_color;
      if ( importance > importance_cutoff ) {
        color = TraceRay(bhp, t, depth+1, importance);
      }
      result += (1.0f - reflection) * refraction_color * color;
    }
    // else TIR
  } // else reflection==1 so refraction has 0 weight

  // reflection
  float3 color = cutoff_color;
  if (depth < min(reflection_maxdepth, max_depth))
  {
    r = reflect(i, n);
  
    float importance = prd_radiance.importance * reflection * optix::luminance( reflection_color * beer_attenuation );
    if ( importance > importance_cutoff ) {
      color = TraceRay( fhp, r, depth+1, importance );
    }
  }
  result += reflection * reflection_color * color;

  result = result * beer_attenuation;

  return result;
}

