// types

enum RTtransformkind {
    RT_WORLD_TO_OBJECT = 0xf00,
    RT_OBJECT_TO_WORLD
};
typedef enum RTtransformkind RTtransformkind;

struct float3
{
    float x, y, z;
};
typedef struct float3 float3;

struct PerRayData_shadow
{
    float3 attenuation;
};
typedef struct PerRayData_shadow PerRayData_shadow;

struct Ray {
    float3 origin; 
    float3 direction;
    unsigned int ray_type;
    float tmin;
    float tmax;
};
typedef struct Ray Ray;

// function signatures

float dot(float3 a, float3 b);
float fabs(float x);
float3 fresnel_schlick(float cos_theta, float exponent, float3 minimum, float3 maximum);
float3 make_float3(float s);
void mul_eq(float3, float3);
float3 normalize(float3 v);
float optix_cc_luminance(float3 rgb);
void rtIgnoreIntersection();
void rtTerminateRay();
float3 rtTransformNormal( RTtransformkind kind, float3 n );
float3 sub13(float, float3);

// global variables

extern float importance_cutoff;
extern PerRayData_shadow prd_shadow;
extern Ray ray;
extern float3 shadow_attenuation;
extern float3 shading_normal;

// -----------------------------------------------------------------------------

__attribute__((__kernelfun__, __global__)) void any_hit_shadow()
{
    float3 world_normal = normalize( rtTransformNormal( RT_OBJECT_TO_WORLD, shading_normal ) );
    float nDi = fabs(dot(world_normal, ray.direction));

    mul_eq(prd_shadow.attenuation, sub13(1,fresnel_schlick(nDi, 5, sub13(1,shadow_attenuation), make_float3(1))));
    if(optix_cc_luminance(prd_shadow.attenuation) < importance_cutoff)
        rtTerminateRay();
    else
        rtIgnoreIntersection();
}
