#!/usr/bin/python

import glob
import os
from subprocess import call, Popen, PIPE
import re
import shutil
import sys

if len( sys.argv ) != 4:
    print "Usage:", sys.argv[ 0 ], "piece variant outfile"
    exit( 0 )

piece   = sys.argv[ 1 ]
variant = sys.argv[ 2 ]
outfile = sys.argv[ 3 ]

strip = os.path.join( os.path.dirname( sys.argv[ 0 ] ), "strip.py" )

def dumpSource( sourcefile, suffix, outfile ):
    """
    De-mangles the source file and renames its kernel with the given suffix.
    The result is written to a file. This frees up the original name for our
    wrapper function, which will call this renamed function when appropriate.
    """

    p = Popen( [ strip, sourcefile ], stdout=PIPE )
    text = p.communicate()[ 0 ]
    if p.returncode != 0:
        print >>sys.stderr, "ERROR:", strip, sourcefile
        exit( 1 )

    signature_pat = \
        re.compile( "((\w+ +)?__device__ +__inline__ +(\w+ +)?\w+)\((void)?\)" )

    fun = None
    for line in text.split( "\n" ):
        match = signature_pat.match( line )
        if match:
            fun = match.group( 1 )
            line = signature_pat.sub( fun + suffix + "(void)", line )
        print >>outfile, line
    return fun

def combineSources( variant ):
    ver = re.search( "(\d+)", os.path.basename( variant ) )
    if ver is None:
        raise Exception(
            "could not extract version number from variant: " + variant
        )
    ver = str( int( ver.group( 0 ) ) )

    base = os.path.splitext( piece )[ 0 ]
    components = set()
    for fname in glob.glob( base.strip( "0123456789" ) + "*" ):
        fname = os.path.splitext( fname )[ 0 ]
        if fname + ".c" == piece:
            components.add( fname + ".c" )
        elif os.path.exists( fname + ".cu" ):
            components.add( fname + ".cu" )
    components = sorted( components )

    with open( outfile, 'w' ) as fh:
        fun = None
        for component in components:
            if component == piece:
                fun = dumpSource( piece, "_golden", fh )
                tmp = dumpSource( variant, ver, fh )
                if fun == None:
                    fun = tmp
                if fun == None:
                    print >>sys.stderr, "ERROR: did not detect function name"
                    exit( 1 )

                funname = fun.split()[ -1 ]
                print >>fh, fun, "() {"
                print >>fh, "if ( prd_radiance.importance > threshold )"
                print >>fh, "    return %s%s();" % ( funname, "_golden" )
                print >>fh, "else"
                print >>fh, "    return %s%s();" % ( funname, ver )
                print >>fh, "}"
            else:
                with open( component ) as f:
                    shutil.copyfileobj( f, fh )

combineSources( variant )

