#!/usr/bin/Rscript

args <- commandArgs( TRUE )
if ( length( args ) != 2 )
    stop( "usage: showsweep.R csvfile pdffile" )

pdffile <- args[ 2 ]

csv <- read.csv( args[ 1 ] )
variants <- unique( csv$variant )

if ( "original" %in% variants ) {
    has_original <- TRUE
    variants <- variants[ variants != "original" ]
    orig_time <- csv$time[ csv$variant == "original" ]
    orig_err  <- csv$error[ csv$variant == "original" ]
} else {
    has_original <- FALSE
}
threshs  <- unique( csv$threshold[ csv$variant %in% variants ] )

times <- matrix( NA, nrow = length( threshs ), ncol = length( variants ) )
errs  <- matrix( NA, nrow = length( threshs ), ncol = length( variants ) )
for ( i in 1:length( variants ) ) {
    for ( j in 1:length( threshs ) ) {
        index <- csv$variant == variants[ i ] & csv$threshold == threshs[ j ]
        times[ j, i ] <- csv$time[ index ]
        errs[ j, i ]  <- csv$error[ index ]
    }
}

pdf( pdffile )
plot( NA,
    xlab = "Error (L2 in RGB)",
    ylab = "Rendering Time (s)",
    xlim = range( errs ), ylim = c( 0, max( times ) )
)
cols <- rainbow( length( variants ) )
ltys <- rep( 1, length( variants ) )
pchs <- rep( 1, length( variants ) )
for ( i in 1:length( variants ) ) {
    lines( errs[ ,i ], times[ ,i ], col = cols[ i ] )
    points( errs[ ,i ], times[ ,i ], col = cols[ i ], cex = 0.75 )
}
if ( has_original ) {
    variants <- c( as.character( variants ), "original" )
    cols <- c( cols, gray( 0 ) )
    ltys <- c( ltys, 2 )
    pchs <- c( pchs, NA )
    abline( h = orig_time, lty = 2, col = gray( 0 ) )
}
legend( "topright",
    legend = basename( variants ), col = cols, pch = pchs, lty = ltys,
    lwd = 1, pt.cex = 0.75, bg = gray( 1 )
)
dev.off()

