#!/usr/bin/python

import re
import sys

# De-mangles a "C"-style file back into a CUDA file. The CUDA code is printed
# to STDOUT.
#
# Note: This script is intended as a utility for compile.sh and combine.py and
# isn't particularly useful to run standalone.

attr_pat = re.compile( "__attribute__\(\(([^\)]*)\)\)" )

source = open( sys.argv[ 1 ] )

# Skip the file content before the first __kernelfun__ attribute. This is just
# a header that was added to allow CIL to parse the CUDA types in the mangled
# file.

for line in source:
    match = attr_pat.search( line )
    if match == None:
        continue
    terms = match.group( 1 ).split( ", " )
    terms2 = filter( lambda x: x != "__kernelfun__", terms )
    if len( terms ) != len( terms2 ):
        if "__global__" in terms2:
            print attr_pat.sub( "RT_PROGRAM", line, count=1 ),
        else:
            print attr_pat.sub( "__device__ __inline__", line, count=1 ),
        break

# Since we know the original file was split to have exactly one kernel per
# segment, we can strip the rest of the attributes. Also, from here on, we
# need to restore the namespace separators.

attr_pat = re.compile( "\([^\(]*__attribute__\(\([^\)]*\)\)[^\)]*\)" )
namespace_pat = re.compile( "_cc_" );
for line in source:
    line = attr_pat.sub( "", line );
    line = namespace_pat.sub( "::", line )
    print line,

