#!/usr/bin/python

# Evaluates each given variant as a low-importance shader paired with the
# original shader, using a range of transition thresholds. These are recorded
# in a CSV file named sweep.csv. If all the variants are in the same directory,
# the CSV file will be placed in the same directory; otherwise, it will be
# placed in the current directory.

import csv
import glob
import os
from subprocess import call, check_call, Popen, PIPE, CalledProcessError
import re
import sys

if len( sys.argv ) < 3:
    print "Usage:", sys.argv[ 0 ], "piece-file variants..."
    exit( 0 )

piece    = sys.argv[ 1 ]
variants = sys.argv[ 2: ]

csvfile  = set( map( os.path.dirname, variants ) )
if len( csvfile ) == 1:
    csvfile = os.path.join( csvfile.pop(), "sweep.csv" )
else:
    csvfile = "sweep.csv"

test    = os.path.join( os.path.dirname( sys.argv[ 0 ] ), "test.py" )
combine = os.path.join( os.path.dirname( sys.argv[ 0 ] ), "combine.py" )
compile = os.path.join( os.path.dirname( sys.argv[ 0 ] ), "compile.sh" )

def quiet_check( *args, **kwargs ):
    try:
        check_call( *args, **kwargs )
    except CalledProcessError as e:
        exit( e.returncode )

def get_fitness( threshold = None ):
    if threshold is None:
        lines = Popen( [ test ], stdout = PIPE ).communicate()[ 0 ]
    else:
        lines = Popen(
            [ test, "-t", threshold ], stdout = PIPE
        ).communicate()[ 0 ]
    fitness = None
    for line in lines.split( "\n" ):
        line = line.split()
        if len( line ) > 0:
            fitness = line[ :2 ]
    return fitness

with open( csvfile, 'w' ) as fh:
    writer = csv.writer( fh )
    writer.writerow( [ "variant", "threshold", "time", "error" ] )
    quiet_check( [ compile, piece, piece ] )
    fitness = get_fitness()
    if fitness is not None:
        writer.writerow( [ "original", "" ] + fitness )
    for variant in variants:
        quiet_check( [ combine, piece, variant, "src/glass.cu" ] )
        quiet_check( [ "make", "-C", "src" ] )
        for threshold in range( 21 ):
            threshold = str( float( threshold ) / 20.0 )
            fitness = get_fitness( threshold )
            if fitness is not None:
                writer.writerow( [ variant, threshold ] + fitness )
                fh.flush()

